
#ifndef __LOOPPROGS__INSTRUCTIONS_H
    #define __LOOPPROGS__INSTRUCTIONS_H


    #include <string>
    #include <map>
    #include <vector>


    class ExecutionContext
    {
        public:
            std::map<std::string, int> Variables;
    };



    class LoopInstruction;



    class LoopProgram
    {
        public:
            LoopProgram();
            ~LoopProgram();
            std::vector<LoopInstruction*> Instructions;
            void Execute(ExecutionContext* context);
    };



    class LoopInstruction
    {
        public:
            LoopInstruction();
            virtual ~LoopInstruction();
            virtual void Execute(ExecutionContext* context) = 0;
    };



    class LoopVarLoop : public LoopInstruction
    {
        public:
            LoopProgram* Block;
            std::string IterVar;

            LoopVarLoop(std::string IterVar, LoopProgram* Block);
            ~LoopVarLoop();
            virtual void Execute(ExecutionContext* context);
    };


    class LoopConstLoop : public LoopInstruction
    {
        public:
            LoopProgram* Block;
            int IterCount;

            LoopConstLoop(int IterCount, LoopProgram* Block);
            ~LoopConstLoop();
            virtual void Execute(ExecutionContext* context);
    };


    class Assignment : public LoopInstruction
    {
        public:
            std::string DstVariable;
            std::string SrcVariable;
            int Added;

            Assignment(std::string DstVariable, std::string SrcVariable, int Added);
            ~Assignment();
            virtual void Execute(ExecutionContext* context);
    };


#endif