%option c++
%option yyclass="LoopLexer"
%option noyywrap
%option nostdinit
%option yylineno
%option case-insensitive
%option full
%option prefix="LOOP_"

%{
    #include <iostream>
    #include <sstream>
    #include "parser.hpp"
    #include "looplexer.h"
%}
%% // --------------------------------------------------------------------------
"//"[^\n]*\n                            {}
"/*"([^\*]*(\*+)[^*/])*([^\*]*)(\*+)"/" {}
[\n\t" "]*                              {}
";"                                     {return yy::LoopParser::token::SEMICOLON;}
"loop"                                  {return yy::LoopParser::token::LOOP;}
"do"                                    {return yy::LoopParser::token::DO;}
"done"                                  {return yy::LoopParser::token::DONE;}
":="                                    {return yy::LoopParser::token::ASSIGN;}
"+"                                     {return yy::LoopParser::token::ADD;}
"-"                                     {return yy::LoopParser::token::SUB;}
[a-zA-Z][a-zA-Z0-9]*                    {
                                          yylval->stringval = new std::string(yytext);
                                          return yy::LoopParser::token::ID;
                                        }
[0-9]+                                  {
                                          yylval->intval = new int;
                                          std::stringstream s;
                                          s << yytext;
                                          s >> *(yylval->intval);
                                          return yy::LoopParser::token::NUM;
                                        }
.|\n                                    {
                                          std::cerr << "\n\nLEXICAL ERROR (Line "
                                            << yylineno << "): Unexpected character \""
                                            << yytext[0] << "\"\n";
                                        }
%% //---------------------------------------------------------------------------

LoopLexer::LoopLexer(std::istream& is)
    : LOOP_FlexLexer(&is)
{
}

int LoopLexer::yylex(yy::LoopParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}

int LoopLexer::GetCurrentLineNumber()
{
    return yylineno;
}
